<?php

namespace UNIFIED_SOFTWARE\EMAILVAL;

add_action( 'admin_menu', __NAMESPACE__ . '\unified_software_emailval_add_admin_menu' );
add_action( 'admin_init', __NAMESPACE__ . '\unified_software_emailval_init' );

function unified_software_emailval_add_admin_menu() {
    add_submenu_page(
        'unified_software',
        'EmailVal Settings',
        'EmailVal',
        'manage_options',
        'unified_software_emailval',
        __NAMESPACE__ . '\unified_software_emailval_options_page'
    );
}

function unified_software_emailval_init() {
    register_setting( 'emailval', 'unified_software_emailval' );

    add_settings_section(
        'unified_software_emailval_general_section',
        'EmailVal General Settings',
        __NAMESPACE__ . '\emailval_general_section_callback',
        'emailval'
    );

    add_settings_field(
        'email_verification',
        __( 'email verification', 'unified_software' ),
        __NAMESPACE__ . '\emailval_email_verification_field_render',
        'emailval',
        'unified_software_emailval_general_section'
    );

    add_settings_field(
        'polling_interval',
        __( 'polling interval', 'unified_software' ),
        __NAMESPACE__ . '\emailval_polling_interval_field_render',
        'emailval',
        'unified_software_emailval_general_section'
    );

    add_settings_section(
        'unified_software_emailval_mysql_section',
        'EmailVal MySQL Settings',
        __NAMESPACE__ . '\emailval_mysql_section_callback',
        'emailval'
    );

    add_settings_field(
        'mysql_table',
        __( 'table', 'unified-software' ),
        __NAMESPACE__ . '\emailval_mysql_table_field_render',
        'emailval',
        'unified_software_emailval_mysql_section'
    );

    add_settings_section(
        'unified_software_emailval_misc_section',
        __( 'EmailVal Miscellaneous Settings', 'unified-software' ),
        __NAMESPACE__ . '\emailval_misc_section_callback',
        'emailval'
    );

    add_settings_field(
        'misc_powered_by',
        __( 'show "powered by" message under EmailVal form' ),
        __NAMESPACE__ . '\emailval_misc_powered_by_field_render',
        'emailval',
        'unified_software_emailval_misc_section'
    );
}

function unified_software_emailval_options_page() {
    if ( ! current_user_can( 'manage_options' ) ) {
        wp_die( '<h1>You lack permission to manage_options</h1>' );
    }

    ?>
    <form action="options.php" method="post">
        <?php
            settings_fields( 'emailval' );
            do_settings_sections( 'emailval' );
            submit_button();
        ?>
    </form>
    <?php
}

// Section Callbacks

function emailval_general_section_callback() {
    echo 'Configure general EmailVal settings';
}

function emailval_mysql_section_callback() {
    echo 'Set up your preferred MySQL storage location for validated email addresses';
    $option = get_option( 'unified_software_emailval_mysql_table_error' );
    if ( ! $option ) {
        return;
    }
    ?>
    <p style='color: red'><?php echo esc_html( $option ) ?></p>
    <?php
    delete_option( 'unified_software_emailval_mysql_table_error' ); 
}

function emailval_misc_section_callback() {
    echo 'Configure additional options';  
}

// Field Callbacks

function emailval_email_verification_field_render() {
    $name = 'email_verification';
    $option = get_option( 'unified_software_emailval' );
    $value = $option && array_key_exists( $name, $option ) ? $option[$name] : '';
    ?>
        <input id='<?php echo esc_attr( $name ) ?>'
            name='<?php echo sprintf( 'unified_software_emailval[%s]', esc_attr( $name ) ) ?>'
            type='checkbox' value='1' <?php echo checked( 1, $value, false ) ?>
        >
    <?php
}

function emailval_polling_interval_field_render() {
    $name = 'polling_interval';
    $option = get_option( 'unified_software_emailval' );
    $value = $option && array_key_exists( $name, $option ) ? $option[$name] : '';
    ?>
        <p>
            <input type='radio' value='every_5_minutes' id="radio_interval_5_mins"
                name='<?php echo sprintf( 'unified_software_emailval[%s]', esc_attr( $name ) ) ?>'
                <?php echo checked( 'every_5_minutes', $value, false ) ?>
            >
            <label for="radio_interval_5_mins">every 5 minutes</label>
        </p>

        <p>
            <input type='radio' value='hourly' id="radio_interval_hourly"
                name='<?php echo sprintf( 'unified_software_emailval[%s]', esc_attr( $name ) ) ?>'
                <?php echo checked( 'hourly', $value, false ) ?>
            >
            <label for="radio_interval_hourly">hourly</label>
        </p>

        <p>
            <input type='radio' value='daily' id="radio_interval_daily"
                name='<?php echo sprintf( 'unified_software_emailval[%s]', esc_attr( $name ) ) ?>'
                <?php echo checked( 'daily', $value, false ) ?>
            >
            <label for="radio_interval_daily">daily</label>
        </p>
    <?php
}

function emailval_mysql_table_field_render() {
    $name = 'mysql_table';
    $option = get_option( 'unified_software_emailval' );
    $value = $option && array_key_exists( $name, $option ) ? $option[$name] : '';
    if ( $value != '' && ( ! does_table_exist( $value ) || ! is_emailval_table( $value ) ) ) {
        $value = '';
        unset( $option[$name] );
        update_option( 'unified_software_emailval', $option );
    }
    ?>
        <input title='letters or underscores' id='<?php echo esc_attr( $name ) ?>' pattern='[a-z,A-Z,_]+'
            name='<?php echo sprintf( 'unified_software_emailval[%s]', esc_attr( $name ) ) ?>'
            type='text' value='<?php echo esc_attr( $value ) ?>'
        >
    <?php
}

function emailval_misc_powered_by_field_render() {
    $name = 'misc_powered_by';
    $option = get_option( 'unified_software_emailval' );
    $value = $option && array_key_exists( $name, $option ) ? $option[$name] : '';
    ?>
        <input id='<?php echo esc_attr( $name ) ?>' type='checkbox' value='1'
            name='<?php echo sprintf( 'unified_software_emailval[%s]', esc_attr( $name ) ) ?>'
            <?php echo checked( 1, $value, false ) ?>
        >
    <?php
}

add_action( 'update_option_unified_software_emailval', function( $oldValue, $value ) {
    if ( $oldValue['polling_interval'] != $value['polling_interval'] ) {
        wp_clear_scheduled_hook( 'emailval_poll_verify' );
        schedule_email_verification_polling();
    }
}, 10, 2 );

add_filter( 'pre_update_option_unified_software_emailval', function( $value, $oldValue ) {
    foreach ( $value as $k => $v ) {
        $v = sanitize_text_field( $v );
        if ( 'mysql_table' == $k && $v != '' ) {
            if ( ! does_table_exist( $v ) ) {
                create_emailval_table( $v );
            } else if ( ! is_emailval_table( $v ) ) {
                $value[$k] = $oldValue[$k];
                add_option(
                    'unified_software_emailval_mysql_table_error',
                    "ERROR: TABLE ${v} ALREADY EXISTS AND DOES NOT MATCH EXPECTED STRUCTURE"
                );
            }
        }
    }

    return $value;
}, 10, 2 );

?>